Attribute VB_Name = "MRunDialog"
' *************************************************************************
'  Copyright 2000 Karl E. Peterson
'  All Rights Reserved, http://www.mvps.org/vb
' *************************************************************************
'  You are free to use this code within your own applications, but you
'  are expressly forbidden from selling or otherwise distributing this
'  source code, non-compiled, without prior written consent.
' *************************************************************************
Option Explicit

' This module shows two ways to invoke the common Run dialog.
' The first is the "sanctioned" method, which uses the Shell
' automation object's FileRun method, then hijacks the dialog
' to replace most of the elements -- caption, icon, position,
' etc...
Private Declare Function SetTimer Lib "user32" (ByVal hWnd As Long, ByVal nIDEvent As Long, ByVal uElapse As Long, ByVal lpTimerFunc As Long) As Long
Private Declare Function KillTimer Lib "user32" (ByVal hWnd As Long, ByVal nIDEvent As Long) As Long
Private Declare Function FindWindow Lib "user32" Alias "FindWindowA" (ByVal lpClassName As String, ByVal lpWindowName As String) As Long
Private Declare Function ShowWindow Lib "user32" (ByVal hWnd As Long, ByVal nCmdShow As Long) As Long
Private Declare Function GetWindow Lib "user32" (ByVal hWnd As Long, ByVal wCmd As Long) As Long
Private Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hWnd As Long, ByVal wMsg As Long, ByVal wParam As Long, lParam As Any) As Long
Private Declare Function GetWindowRect Lib "user32" (ByVal hWnd As Long, lpRect As RECT) As Long
Private Declare Function MoveWindow Lib "user32" (ByVal hWnd As Long, ByVal x As Long, ByVal y As Long, ByVal nWidth As Long, ByVal nHeight As Long, ByVal bRepaint As Long) As Long
Private Declare Function SetForegroundWindow Lib "user32" (ByVal hWnd As Long) As Long

Private Type RECT
   Left As Long
   Top As Long
   Right As Long
   Bottom As Long
End Type

Private Const GW_HWNDNEXT = 2
Private Const GW_CHILD = 5

Private Const WM_SETTEXT = &HC
Private Const STM_SETIMAGE = &H172

Private Const IMAGE_BITMAP = 0
Private Const IMAGE_ICON = 1
Private Const IMAGE_CURSOR = 2
Private Const IMAGE_ENHMETAFILE = 3

Private m_TimerID As Long
Private m_hWndOwner As Long
Private m_Left As Long
Private m_Top As Long
Private m_hIcon As Long
Private m_Caption As String
Private m_Desc As String

' The second method uses an undocumented export from
' shell32.dll to do what NT's TaskMgr.exe does directly.
' This techique is fully documented on James Holderness'
' "Undocumented Windows 95" website:
' http://www.geocities.com/SiliconValley/4942/common.html
Private Declare Function RunFileDlgNT Lib "shell32" Alias "#61" (ByVal hWndOwner As Long, ByVal hIcon As Long, ByVal lpstrDirectory As Long, ByVal lpstrTitle As Long, ByVal lpstrDescription As Long, ByVal uFlags As Long) As Long
Private Declare Function RunFileDlg9x Lib "shell32" Alias "#61" (ByVal hWndOwner As Long, ByVal hIcon As Long, ByVal lpstrDirectory As String, ByVal lpstrTitle As String, ByVal lpstrDescription As String, ByVal uFlags As Long) As Long

Private Const RFF_NOBROWSE As Long = &H1         ' Removes the browse button.
Private Const RFF_NODEFAULT As Long = &H2        ' No default item selected.
Private Const RFF_CALCDIRECTORY As Long = &H4    ' Calculates the working directory from the file name.
Private Const RFF_NOLABEL As Long = &H8          ' Removes the edit box label.
Private Const RFF_NOSEPARATEMEM As Long = &H20   ' Removes the Separate Memory Space check box(Windows NT only).

Private Declare Function GetVersionEx Lib "kernel32" Alias "GetVersionExA" (lpVersionInformation As OSVERSIONINFO) As Long
Private Type OSVERSIONINFO
   dwOSVersionInfoSize As Long
   dwMajorVersion As Long
   dwMinorVersion As Long
   dwBuildNumber As Long
   dwPlatformId As Long
   szCSDVersion As String * 128
End Type
Private Const VER_PLATFORM_WIN32s = 0
Private Const VER_PLATFORM_WIN32_WINDOWS = 1
Private Const VER_PLATFORM_WIN32_NT = 2

Public Sub ShowRunDialog2(Optional ByVal hWndOwner As Long = 0, _
   Optional ByVal hIcon As Long = 0, _
   Optional ByVal Caption As String = "", _
   Optional ByVal Description As String = "", _
   Optional ByVal Directory As String = "")
   ' The RunFileDlg call is OS-specific!
   ' ANSI only in 9x, and Unicode only in NT.
   Dim os As OSVERSIONINFO
   Dim lpTitle As Long
   Dim lpDescription As Long
   Dim lpDirectory As Long
   
   os.dwOSVersionInfoSize = Len(os)
   Call GetVersionEx(os)
   If os.dwPlatformId = VER_PLATFORM_WIN32_NT Then
      If Len(Caption) Then lpTitle = StrPtr(Caption)
      If Len(Description) Then lpDescription = StrPtr(Description)
      If Len(Directory) Then lpDirectory = StrPtr(Directory)
      Call RunFileDlgNT(hWndOwner, hIcon, lpDirectory, lpTitle, lpDescription, RFF_CALCDIRECTORY)
   Else
      If Caption = "" Then Caption = vbNullString
      If Description = "" Then Description = vbNullString
      If Directory = "" Then Directory = vbNullString
      Call RunFileDlg9x(hWndOwner, hIcon, vbNullString, Caption, Description, RFF_CALCDIRECTORY)
   End If
End Sub

Public Sub ShowRunDialog1(Optional ByVal hWndOwner As Long = 0, _
   Optional ByVal Left As Long = -1, _
   Optional ByVal Top As Long = -1, _
   Optional ByVal hIcon As Long = 0, _
   Optional ByVal Caption As String = "", _
   Optional ByVal Description As String = "")
   Dim so As Object

   ' Assign passed values to module vars
   m_hWndOwner = hWndOwner
   m_Left = Left
   m_Top = Top
   m_hIcon = hIcon
   m_Caption = Caption
   m_Desc = Description
   
   ' Try creating a new Shell instance.
   On Error Resume Next
   Set so = CreateObject("Shell.Application")
   If Err.Number Then
      ' Bummer :-(
      MsgBox "Can't invoke Run dialog via Shell object.", _
              vbExclamation, "IE4 Required"
   Else
      ' Show dialog, set timer, and return.
      so.FileRun
      m_TimerID = SetTimer(0&, 0&, 1&, AddressOf TimerProc)
   End If
End Sub

Private Sub TimerProc(ByVal hWndOwner As Long, ByVal uMsg As Long, ByVal idEvent As Long, ByVal dwTime As Long)
   Dim hWnd As Long
   Dim hWndStatic As Long
   Dim r1 As RECT
   Dim r2 As RECT
   Dim Tries As Long
   
   ' Get handle to dialog.
   hWnd = FindWindow(vbNullString, "Run")
   
   If hWnd Then
      ' Kill timer.
      Call KillTimer(0&, m_TimerID)
      Tries = 0
      
      ' Hide the dialog.
      Call ShowWindow(hWnd, vbHide)
      
      ' Overwrite caption?
      If Len(m_Caption) Then
         Call SendMessage(hWnd, WM_SETTEXT, 0&, ByVal m_Caption)
      End If
      
      ' Overwrite icon?
      hWndStatic = GetWindow(hWnd, GW_CHILD)
      If m_hIcon Then
         Call SendMessage(hWndStatic, STM_SETIMAGE, IMAGE_ICON, ByVal m_hIcon)
      End If
   
      ' Overwrite description?
      hWndStatic = GetWindow(hWndStatic, GW_HWNDNEXT)
      If Len(m_Desc) Then
         Call SendMessage(hWndStatic, WM_SETTEXT, 0&, ByVal m_Desc)
      End If
   
      ' Get coords for dialog.
      Call GetWindowRect(hWnd, r2)
      
      ' Position over requested window,
      ' or to requested location?
      If m_hWndOwner Then
         Call GetWindowRect(m_hWndOwner, r1)
         Call MoveWindow(hWnd, r1.Left + ((r1.Right - r1.Left) \ 2 - (r2.Right - r2.Left) \ 2), _
            r1.Top + (r1.Bottom - r1.Top) \ 2 - (r2.Bottom - r2.Top) \ 2, _
            r2.Right - r2.Left, r2.Bottom - r2.Top, True)
      ElseIf m_Left >= 0 And m_Top >= 0 Then
         Call MoveWindow(hWnd, m_Left, m_Top, r2.Right - r2.Left, r2.Bottom - r2.Top, True)
      End If
      
      ' Bring dialog to front.
      Call ShowWindow(hWnd, vbNormalFocus)
      Call SetForegroundWindow(hWnd)
      
   Else
      ' Try ten(?) times, then kill the timer.
      Tries = Tries + 1
      If Tries >= 10 Then
         Call KillTimer(0&, m_TimerID)
         Tries = 0
      End If
   End If
End Sub
